package com.ycwl.basic.repository;

import com.ycwl.basic.mapper.OrderMapper;
import com.ycwl.basic.model.pc.order.entity.OrderEntity;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.redis.core.RedisTemplate;
import org.springframework.stereotype.Component;

import java.util.concurrent.TimeUnit;

@Component
public class OrderRepository {
    @Autowired
    private OrderMapper orderMapper;
    @Autowired
    private RedisTemplate<String, String> redisTemplate;

    public static final String ORDER_ITEM_CACHE_KEY = "order:user:%s:type:%s:id:%s";

    public boolean checkUserBuyItem(Long userId, int goodsType, Long goodsId) {
        if (redisTemplate.hasKey(String.format(ORDER_ITEM_CACHE_KEY, userId, goodsType, goodsId))) {
            return "1".equals(redisTemplate.opsForValue().get(String.format(ORDER_ITEM_CACHE_KEY, userId, goodsType, goodsId)));
        }
        OrderEntity orderEntity = orderMapper.getUserBuyItem(userId, goodsType, goodsId);
        if (orderEntity == null) {
            redisTemplate.opsForValue().set(String.format(ORDER_ITEM_CACHE_KEY, userId, goodsType, goodsId), "0", 60, TimeUnit.SECONDS);
            return false;
        }
        redisTemplate.opsForValue().set(String.format(ORDER_ITEM_CACHE_KEY, userId, goodsType, goodsId), "1");
        return true;
    }

    public boolean checkUserBuyFaceSourceImage(Long userId, Long faceId) {
        return checkUserBuyItem(userId, 2, faceId);
    }

    public boolean checkUserBuyFaceSourceVideo(Long userId, Long faceId) {
        return checkUserBuyItem(userId, 1, faceId);
    }

    public boolean checkUserBuyVideo(Long userId, Long videoId) {
        return checkUserBuyItem(userId, 0, videoId);
    }

    public void clearUserBuyItemCache(Long userId, int goodsType, Long goodsId) {
        redisTemplate.delete(String.format(ORDER_ITEM_CACHE_KEY, userId, goodsType, goodsId));
    }
}
