package com.ycwl.basic.service.impl.pc;

import com.github.pagehelper.PageHelper;
import com.github.pagehelper.PageInfo;
import com.ycwl.basic.repository.DeviceRepository;
import com.ycwl.basic.mapper.DeviceMapper;
import com.ycwl.basic.model.pc.device.entity.DeviceConfigEntity;
import com.ycwl.basic.model.pc.device.req.DeviceAddOrUpdateReq;
import com.ycwl.basic.model.pc.device.req.DeviceReqQuery;
import com.ycwl.basic.model.pc.device.resp.DeviceRespVO;
import com.ycwl.basic.service.pc.DeviceService;
import com.ycwl.basic.utils.ApiResponse;
import com.ycwl.basic.utils.SnowFlakeUtil;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.List;

/**
 * @Author：longbinbin
 * @Date：2024/12/2 16:18
 */
@Service
public class DeviceServiceImpl implements DeviceService {
    @Autowired
    private DeviceMapper deviceMapper;
    @Autowired
    private DeviceRepository deviceRepository;
    @Override
    public ApiResponse<PageInfo<DeviceRespVO>> pageQuery(DeviceReqQuery deviceReqQuery) {
        PageHelper.startPage(deviceReqQuery.getPageNum(), deviceReqQuery.getPageSize());
        List<DeviceRespVO> list = deviceMapper.list(deviceReqQuery);
        PageInfo<DeviceRespVO> pageInfo = new PageInfo<>(list);
        return ApiResponse.success(pageInfo);
    }

    @Override
    public ApiResponse<List<DeviceRespVO>> list(DeviceReqQuery deviceReqQuery) {
        return ApiResponse.success(deviceMapper.list(deviceReqQuery));
    }

    @Override
    public ApiResponse<DeviceRespVO> getById(Long id) {
        return ApiResponse.success(deviceMapper.getById(id));
    }

    @Override
    public ApiResponse addOrUpdate(DeviceAddOrUpdateReq deviceReqQuery) {
        Long id = deviceReqQuery.getId();
        if (id == null) {
            deviceReqQuery.setId(SnowFlakeUtil.getLongId());
            if (StringUtils.isBlank(deviceReqQuery.getNo())) {
                deviceReqQuery.setNo(deviceReqQuery.getId().toString());
            }
            deviceReqQuery.setStatus(0);
            return ApiResponse.success(deviceMapper.add(deviceReqQuery));
        } else {
            deviceRepository.clearDeviceCache(deviceReqQuery.getId());
            deviceMapper.update(deviceReqQuery);
            deviceRepository.clearDeviceCache(deviceReqQuery.getId());
            return ApiResponse.success(0);
        }
    }

    @Override
    public ApiResponse deleteById(Long id) {
        return ApiResponse.success(deviceMapper.deleteById(id));
    }

    @Override
    public ApiResponse updateStatus(Long id) {
        return ApiResponse.success(deviceMapper.updateStatus(id));
    }

    @Override
    public DeviceConfigEntity getConfig(Long id) {
        DeviceConfigEntity config = deviceMapper.getConfigByDeviceId(id);
        if (config == null) {
            config = new DeviceConfigEntity();
            config.setId(SnowFlakeUtil.getLongId());
            config.setDeviceId(id);
            deviceMapper.addConfig(config);
        }
        return config;
    }

    @Override
    public void saveConfig(Long configId, DeviceConfigEntity config) {
        config.setId(configId);
        deviceMapper.updateConfig(config);
        deviceRepository.clearDeviceCache(config.getDeviceId());
    }
}
