package com.ycwl.basic.service.impl.pc;

import com.github.pagehelper.PageHelper;
import com.github.pagehelper.PageInfo;
import com.ycwl.basic.biz.OrderBiz;
import com.ycwl.basic.constant.BaseContextHandler;
import com.ycwl.basic.constant.NumberConstant;
import com.ycwl.basic.enums.*;
import com.ycwl.basic.exception.AppException;
import com.ycwl.basic.mapper.*;
import com.ycwl.basic.model.mobile.order.IsBuyRespVO;
import com.ycwl.basic.model.mobile.order.OrderAppPageReq;
import com.ycwl.basic.model.mobile.order.PriceObj;
import com.ycwl.basic.model.mobile.order.RefundOrderReq;
import com.ycwl.basic.model.mobile.statistic.req.StatisticsRecordAddReq;
import com.ycwl.basic.model.pc.member.resp.MemberRespVO;
import com.ycwl.basic.model.pc.order.entity.OrderEntity;
import com.ycwl.basic.model.pc.order.entity.OrderItemEntity;
import com.ycwl.basic.model.pc.order.req.CreateOrderReqVO;
import com.ycwl.basic.model.pc.order.req.OrderUpdateReq;
import com.ycwl.basic.model.pc.order.req.OrderReqQuery;
import com.ycwl.basic.model.pc.order.resp.OrderAppRespVO;
import com.ycwl.basic.model.pc.order.resp.OrderItemVO;
import com.ycwl.basic.model.pc.order.resp.OrderRespVO;
import com.ycwl.basic.model.pc.orderOp.entity.OrderOperationEntity;
import com.ycwl.basic.model.pc.source.entity.SourceEntity;
import com.ycwl.basic.model.pc.video.entity.VideoEntity;
import com.ycwl.basic.model.wx.WXPayOrderReqVO;
import com.ycwl.basic.model.wx.WxPayRespVO;
import com.ycwl.basic.service.mobile.GoodsService;
import com.ycwl.basic.service.mobile.WxPayService;
import com.ycwl.basic.service.pc.OrderService;
import com.ycwl.basic.utils.ApiResponse;
import com.ycwl.basic.utils.BigDecimalUtil;
import com.ycwl.basic.utils.SnowFlakeUtil;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.math.BigDecimal;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import java.util.Objects;

/**
 * @Author：longbinbin
 * @Date：2024/12/3 13:54
 */
@Slf4j
@Service
public class OrderServiceImpl implements OrderService {

    @Autowired
    private OrderMapper orderMapper;
    @Autowired
    private VideoMapper videoMapper;
    @Autowired
    private SourceMapper sourceMapper;
    @Autowired
    private WxPayService wxPayService;

    @Autowired
    private MemberMapper memberMapper;
    @Autowired
    private OrderOperationMapper orderOperationMapper;
    @Autowired
    private StatisticsMapper statisticsMapper;
    @Autowired
    private OrderBiz orderBiz;

    @Override
    public ApiResponse<PageInfo<OrderRespVO>> pageQuery(OrderReqQuery query) {
        PageHelper.startPage(query.getPageNum(), query.getPageSize());
        List<OrderRespVO> list = orderMapper.list(query);
        PageInfo<OrderRespVO> pageInfo = new PageInfo<>(list);
        return ApiResponse.success(pageInfo);
    }

    @Override
    public ApiResponse<List<OrderRespVO>> list(OrderReqQuery query) {
        return ApiResponse.success(orderMapper.list(query));
    }

    @Override
    public ApiResponse<OrderRespVO> detail(Long orderId) {
        return ApiResponse.success(orderMapper.getById(orderId));
    }

    /**
     * 发起支付
     * @param order 订单
     * @param orderItems 商品详情
     * @return 支付请求结果
     */
    private  WxPayRespVO initiatePayment(OrderEntity order, List<OrderItemEntity> orderItems) throws Exception {
        WXPayOrderReqVO wxPayOrderReqVO = new WXPayOrderReqVO();
        String goodsName = null;
        if (orderItems.size() > 1) {
            goodsName = "多项景区VLOG商品";
        } else {
            int type = orderItems.get(NumberConstant.ZERO).getGoodsType();
            if (type == NumberConstant.ZERO) {
                goodsName = "景区VLog视频";
            } else if (type == NumberConstant.ONE) {
                goodsName = "景区原片包";
            } else if (type == NumberConstant.TWO) {
                goodsName = "景区照片包";
            }
        }
        if (order.getPayPrice().equals(BigDecimal.ZERO)) {
            // 0元支付
            WxPayRespVO wxPayRespVO = new WxPayRespVO();
            wxPayRespVO.setNeedPay(false);
            return wxPayRespVO;
        }
        wxPayOrderReqVO.setOpenId(order.getOpenId())
                .setMemberId(order.getMemberId())
                .setOrderSn(order.getId())
                .setTotalPrice(BigDecimalUtil.convertToCents(order.getPrice()))
                .setGoodsName(goodsName)
                .setDescription(goodsName);

        return wxPayService.createOrder(wxPayOrderReqVO);
    }

    @Override
    public ApiResponse<Integer> update(OrderUpdateReq query) {
        int update = orderMapper.update(query);
        if (update == NumberConstant.ZERO) {
            return ApiResponse.fail("订单更新失败");
        }
        return ApiResponse.success(update);
    }

    /**
     * 更新订单支付状态
     *
     * @param orderSn 订单编号也就是订单的ID
     */
    @Override
    public void updateOrderState(Long orderSn, OrderStateEnum orderStateEnum, String reason) {
        OrderUpdateReq orderUpdateReq = new OrderUpdateReq();
        orderUpdateReq.setId(orderSn);
        if (orderStateEnum.getType() == NumberConstant.ONE) {//退款状态 0未提出，1已通过，2待审核
            orderUpdateReq.setRefundStatus(orderStateEnum.getState());
            orderUpdateReq.setRefundAt(new Date());
            if (orderStateEnum.getState() == OrderStateEnum.PASSED.getState()) {
                orderUpdateReq.setRemark(reason);
            } else if (Objects.nonNull(reason)) {
                orderUpdateReq.setRefundReason(reason);
            }
        } else if (orderStateEnum.getType() == NumberConstant.TWO) {//订单状态 0未支付，1已支付，2已退款，9已取消
            int state = orderStateEnum.getState();
            orderUpdateReq.setPayAt(new Date());
            orderUpdateReq.setStatus(orderStateEnum.getState());
            if (state == OrderStateEnum.CANCELED.getState()) {//取消支付
                orderUpdateReq.setCancelAt(new Date());

                //如果订单未支付就把商品状态更改成未购买
                OrderRespVO orderRespVO = orderMapper.getById(orderSn);
                Integer goodsType = orderRespVO.getGoodsType();
                List<OrderItemVO> orderItemList = orderRespVO.getOrderItemList();
                orderItemList.forEach(orderItemVO -> {
                    Long goodsId = orderItemVO.getGoodsId();
                    if (Objects.equals(goodsType, GoodsTypeEnum.VIDEO.code)) {
                        VideoEntity videoEntity = new VideoEntity();
                        videoEntity.setId(goodsId);
                        videoEntity.setIsBuy(0);
                        videoMapper.update(videoEntity);
                    } else if (Objects.equals(goodsType, GoodsTypeEnum.SOURCE.code)) {
                        SourceEntity sourceEntity = new SourceEntity();
                        sourceEntity.setId(goodsId);
                        sourceEntity.setIsBuy(0);
                        sourceMapper.update(sourceEntity);
                    }
                });
            } else if (state== OrderStateEnum.PAID.getState()) {//支付成功
                orderUpdateReq.setPayAt(new Date());

            }else if (state == OrderStateEnum.REFUNDED.getState()) {//退款成功
                orderUpdateReq.setRefundAt(new Date());

                //订单退款成功就把商品状态更改成未购买
                OrderRespVO orderRespVO = orderMapper.getById(orderSn);
                Integer goodsType = orderRespVO.getGoodsType();
                List<OrderItemVO> orderItemList = orderRespVO.getOrderItemList();
                orderItemList.forEach(orderItemVO -> {
                    Long goodsId = orderItemVO.getGoodsId();
                    if (Objects.equals(goodsType, GoodsTypeEnum.VIDEO.code)) {
                        VideoEntity videoEntity = new VideoEntity();
                        videoEntity.setId(goodsId);
                        videoEntity.setIsBuy(0);
                        videoMapper.update(videoEntity);
                    } else if (Objects.equals(goodsType, GoodsTypeEnum.SOURCE.code)) {
                        SourceEntity sourceEntity = new SourceEntity();
                        sourceEntity.setId(goodsId);
                        sourceEntity.setIsBuy(0);
                        sourceMapper.update(sourceEntity);
                    }
                });
            }
        }
        orderMapper.update(orderUpdateReq);
    }

    @Override
    public ApiResponse<Integer> getOrderCountByUserId(Long userId) {
        return ApiResponse.success(orderMapper.countByUserId(userId));
    }

    @Override
    public ApiResponse<PageInfo<OrderAppRespVO>> appPageQuery(OrderAppPageReq orderReqQuery) {
        PageHelper.startPage(orderReqQuery.getPageNum(), orderReqQuery.getPageSize());
        List<OrderAppRespVO> list = orderMapper.appList(orderReqQuery);
        PageInfo<OrderAppRespVO> pageInfo = new PageInfo<>(list);
        return ApiResponse.success(pageInfo);
    }

    @Override
    public ApiResponse<OrderAppRespVO> appDetail(Long id) {
        OrderAppRespVO orderAppRespVO = orderMapper.appDetail(id);
        return ApiResponse.success(orderAppRespVO);
    }

    @Override
    public ApiResponse<?> refundOrder(RefundOrderReq refundOrderReq) {
        this.updateOrderState(refundOrderReq.getOrderId(), OrderStateEnum.TO_BE_REVIEWED, refundOrderReq.getRefundReason());
        return ApiResponse.buildResult(BizCodeEnum.REQUEST_OK);
    }

    @Override
    public ApiResponse<?> auditRefundOrder(RefundOrderReq refundOrderReq) {
        this.updateOrderState(refundOrderReq.getOrderId(), OrderStateEnum.PASSED, refundOrderReq.getRefundReason());
        // 调用微信发起退款
        try {
            wxPayService.refundOrder(refundOrderReq.getOrderId().toString());
        } catch (Exception e) {
            throw new AppException(BizCodeEnum.ADVANCE_PAYMENT_REFUND_FAILED,e.getMessage());
        }
        return ApiResponse.buildResult(BizCodeEnum.REQUEST_OK);
    }

    @Override
    public void remarkOrder(Long orderId, OrderEntity query) {
        OrderUpdateReq updateOrder = new OrderUpdateReq();
        updateOrder.setId(orderId);
        updateOrder.setRemark(query.getRemark());
        orderMapper.update(updateOrder);
        OrderOperationEntity orderOperationEntity = new OrderOperationEntity();
        orderOperationEntity.setOrderId(orderId);
        orderOperationEntity.setType("修改备注");
        orderOperationEntity.setRemark("备注修改为："+query.getRemark());
        orderOperationEntity.setUser(BaseContextHandler.getAccount());
        orderOperationEntity.setUserId(BaseContextHandler.getUserId());
        orderOperationMapper.add(orderOperationEntity);
    }

    @Override
    public ApiResponse<PageInfo<OrderRespVO>> refundPageQuery(OrderReqQuery query) {
        PageHelper.startPage(query.getPageNum(), query.getPageSize());
        List<OrderRespVO> list = orderMapper.refundList(query);
        PageInfo<OrderRespVO> pageInfo = new PageInfo<>(list);
        return ApiResponse.success(pageInfo);
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public ApiResponse<WxPayRespVO> createOrder(Long userId, CreateOrderReqVO createOrderReqVO) throws Exception {
        IsBuyRespVO isBuy = orderBiz.isBuy(userId, createOrderReqVO.getScenicId(), createOrderReqVO.getGoodsType(), createOrderReqVO.getGoodsId());
        if (isBuy.isBuy()) {
            return ApiResponse.fail("您已购买此内容，无需重复购买！");
        }
        OrderEntity order = new OrderEntity();
        Long orderId = SnowFlakeUtil.getLongId();
        order.setId(orderId);
        order.setMemberId(userId);
        MemberRespVO member = memberMapper.getById(userId);
        order.setOpenId(member.getOpenId());
        order.setScenicId(createOrderReqVO.getScenicId());
        List<OrderItemEntity> orderItems = new ArrayList<>();
        OrderItemEntity orderItem = new OrderItemEntity();
        orderItem.setGoodsId(createOrderReqVO.getGoodsId());
        orderItem.setGoodsType(createOrderReqVO.getGoodsType());
        orderItem.setOrderId(orderId);
        orderItems.add(orderItem);
        PriceObj priceObj = orderBiz.queryPrice(order.getScenicId(), createOrderReqVO.getGoodsType(), createOrderReqVO.getGoodsId());
        order.setSlashPrice(priceObj.getSlashPrice());
        order.setPrice(priceObj.getPrice());
        // promo code
        // coupon
        order.setPayPrice(priceObj.getPrice());
        if (order.getPayPrice().equals(BigDecimal.ZERO)) {
            order.setStatus(OrderStateEnum.PAID.getState());
            order.setPayAt(new Date());
        } else {
            order.setStatus(OrderStateEnum.UNPAID.getState());
        }
        orderMapper.add(order);
        int addOrderItems = orderMapper.addOrderItems(orderItems);
        if (addOrderItems == NumberConstant.ZERO) {
            log.error("订单明细添加失败");
            return ApiResponse.fail("订单添加失败");
        }
        //点击支付按钮统计
        OrderRespVO orderRespVO = orderMapper.getById(orderId);
        StatisticsRecordAddReq statisticsRecordAddReq = new StatisticsRecordAddReq();
        statisticsRecordAddReq.setMemberId(orderRespVO.getMemberId());
        statisticsRecordAddReq.setType(StatisticEnum.CLICK_ON_PAYMENT.code);
        statisticsRecordAddReq.setScenicId(orderRespVO.getScenicId());
        statisticsRecordAddReq.setMorphId(orderId);
        statisticsMapper.addStatisticsRecord(statisticsRecordAddReq);


        WxPayRespVO wxPayRespVO = initiatePayment(order, orderItems);
        return ApiResponse.success(wxPayRespVO);
    }

}
